@@ -6,12 +6,13 @@ from typing import Any, Dict, Optional, cast
 from urllib.parse import urlparse
 
 import click
+from tqdm import tqdm
+
 from splitgraph.__version__ import __version__
 from splitgraph.config import CONFIG, SG_CMD_ASCII, get_singleton
 from splitgraph.config.management import patch_and_save_config
 from splitgraph.exceptions import DockerUnavailableError, EngineSetupError
 from splitgraph.utils.docker import copy_to_container, get_docker_client
-from tqdm import tqdm
 
 DEFAULT_ENGINE = "default"
 
@@ -103,9 +104,10 @@ def list_engines_c(include_all):
     (whose names start with `splitgraph_engine_`. To manage other engines,
     use Docker CLI directly.
     """
-    from splitgraph.config import CONFIG
     from tabulate import tabulate
 
+    from splitgraph.config import CONFIG
+
     containers = list_engines(include_all=include_all, prefix=CONFIG["SG_ENGINE_PREFIX"])
     if containers:
         our_containers = []
@@ -195,10 +197,23 @@ def _update_bar(
     help="Set the engine as the default engine in the config regardless of its name",
     is_flag=True,
 )
+@click.option(
+    "--cap-add", help="Add kernel capabilities to the engine container", multiple=True, default=[]
+)
 @click.argument("name", default=DEFAULT_ENGINE)
 @click.password_option()
 def add_engine_c(
-    image, port, username, no_init, no_sgconfig, inject_source, no_pull, name, password, set_default
+    image,
+    port,
+    username,
+    no_init,
+    no_sgconfig,
+    inject_source,
+    no_pull,
+    name,
+    password,
+    set_default,
+    cap_add,
 ):
     """
     Create and start a Splitgraph engine.
@@ -214,6 +229,7 @@ def add_engine_c(
     """
     import docker
     from docker.types import Mount
+
     from splitgraph.config import CONFIG
     from splitgraph.engine.postgres.engine import PostgresEngine
 
@@ -260,6 +276,7 @@ def add_engine_c(
                 # Actual config to be injected later
                 "SG_CONFIG_FILE": "/.sgconfig",
             },
+            cap_add=cap_add,
         )
     except docker.errors.APIError as e:
         if "port is already allocated" in str(e):
