@@ -3,12 +3,12 @@ sgr commands related to getting information out of / about images
 """
 
 from collections import Counter, defaultdict
-from typing import List, Optional, Tuple, Union, Dict, cast, TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Dict, List, Optional, Tuple, Union, cast
 
 import click
 
-from .common import ImageType, RepositoryType, remote_switch_option
 from ..core._drawing import format_image_hash, format_tags, format_time
+from .common import ImageType, RepositoryType, emit_sql_results, remote_switch_option
 
 if TYPE_CHECKING:
     from splitgraph.core.repository import Repository
@@ -30,10 +30,12 @@ def log_c(image_spec, tree):
     If ``-t`` or ``--tree`` is passed, this instead renders the full image tree. The repository doesn't need to have
     been checked out in this case.
     """
-    from splitgraph.core._drawing import render_tree
-    from ..core.output import truncate_line
     from tabulate import tabulate
 
+    from splitgraph.core._drawing import render_tree
+
+    from ..core.output import truncate_line
+
     repository, hash_or_tag = image_spec
 
     if tree:
@@ -266,11 +268,12 @@ def object_c(object_id):
     the smallest and largest values for every column are stored in the fragment's metadata. This information is used
     to choose which objects to download in order to execute a query against a table.
     """
+    from splitgraph.core.indexing.bloom import describe
     from splitgraph.core.object_manager import ObjectManager
-    from splitgraph.engine import get_engine, ResultShape
+    from splitgraph.engine import ResultShape, get_engine
+
     from ..core.output import pretty_size
     from ..core.sql import select
-    from splitgraph.core.indexing.bloom import describe
 
     object_manager = ObjectManager(get_engine())
     object_meta = object_manager.get_object_meta([object_id])
@@ -346,18 +349,6 @@ def objects_c(local):
     click.echo("\n".join(sorted(objects)))
 
 
-def _to_str(results: List[Tuple[Any]], use_json: bool = False) -> str:
-    if use_json:
-        import json
-        from splitgraph.core.common import coerce_val_to_json
-
-        return json.dumps(coerce_val_to_json(results))
-
-    from tabulate import tabulate
-
-    return tabulate(results, tablefmt="plain")
-
-
 @click.command(name="sql")
 @click.argument("sql")
 @remote_switch_option()
@@ -413,20 +404,14 @@ def sql_c(sql, schema, image, show_all, json, no_transaction):
         with image.query_schema() as s:
             results = engine.run_sql_in(s, sql)
 
-    if results is None:
-        return
-
-    if len(results) > 10 and not show_all:
-        click.echo(_to_str(results[:10], json))
-        if not json:
-            click.echo("...")
-    else:
-        click.echo(_to_str(results, json))
+    emit_sql_results(results, use_json=json, show_all=show_all)
 
 
 def _emit_repository_data(repositories, engine):
-    from splitgraph.engine import ResultShape
     from tabulate import tabulate
+
+    from splitgraph.engine import ResultShape
+
     from ..core.output import pretty_size
 
     click.echo("Local repositories: \n")
