@@ -7,6 +7,7 @@ from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Tuple, cas
 
 import psycopg2
 from psycopg2.sql import SQL, Identifier
+
 from splitgraph.core.types import (
     Credentials,
     IntrospectionResult,
@@ -164,6 +165,7 @@ class ForeignDataWrapperDataSource(MountableDataSource, LoadableDataSource, ABC)
 
         # Local import here since this data source gets imported by the commandline entry point
         import jsonschema
+
         from splitgraph.core.common import get_temporary_table_id
 
         tmp_schema = get_temporary_table_id()
@@ -473,6 +475,8 @@ If a dictionary, must have the format
                     "options": {[get passed to CREATE FOREIGN TABLE]}}}.
     """
 
+    _icon_file = "postgresql.svg"
+
     def get_server_options(self):
         return {
             "host": self.params["host"],
@@ -536,6 +540,8 @@ Mounts one or more collections on a remote Mongo database as a set of foreign ta
 ```
 """
 
+    _icon_file = "mongodb.svg"
+
     @classmethod
     def get_name(cls) -> str:
         return "MongoDB"
@@ -592,6 +598,8 @@ If a dictionary, must have the format
                     "options": {[get passed to CREATE FOREIGN TABLE]}}}.
         """
 
+    _icon_file = "mysql.svg"
+
     @classmethod
     def get_name(cls) -> str:
         return "MySQL"
@@ -679,8 +687,9 @@ This uses a fork of postgres-elasticsearch-fdw behind the scenes. You can add a
 `query` to your table and set it as `query_column` to pass advanced ES queries and aggregations.
 For example:
 
+\b
 ```
-sgr mount elasticsearch target_schema -c elasticsearch:9200 -o@- <<EOF
+$ sgr mount elasticsearch target_schema -c elasticsearch:9200 -o@- <<EOF
     {
       "tables": {
         "table_1": {
@@ -700,9 +709,12 @@ sgr mount elasticsearch target_schema -c elasticsearch:9200 -o@- <<EOF
       }
     }
 EOF
+\b
 ```
 """
 
+    _icon_file = "elasticsearch.svg"
+
     @classmethod
     def get_name(cls) -> str:
         return "Elasticsearch"
